package net.ihe.gazelle.goctests.interlay;

import net.ihe.gazelle.goctests.interlay.exceptions.ProcessExecutionException;
import net.ihe.gazelle.goctests.application.exceptions.ValidatorGenerationException;
import net.ihe.gazelle.goctests.application.models.GeneratedValidator;
import net.ihe.gazelle.goctests.application.ValidatorGenerator;
import net.ihe.gazelle.lib.hl7templatespackager.application.ProjectNaming;
import net.ihe.gazelle.lib.hl7templatespackager.application.ValidatorConfiguration;
import net.ihe.gazelle.lib.hl7templatespackager.peripherals.HL7Templates2GOC;
import net.ihe.gazelle.goctests.interlay.utils.ValidatorGeneratorUtils;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Path;


public class ValidatorGeneratorImpl implements ValidatorGenerator {


    @Override
    public GeneratedValidator generateValidator(String bbr, String outputFolder, String mvn) throws ValidatorGenerationException {

        System.out.println("Start generation for test");

        HL7Templates2GOC hl7Templates2GOC = new HL7Templates2GOC();
        String[] args = {"-bbr",bbr,"-out",outputFolder,"-mvn",mvn};

        //Run GOC
        boolean validatorGenerated = hl7Templates2GOC.execute(args);

        StringBuilder logPath = new StringBuilder(outputFolder)
                .append("/").append(GeneratedValidator.ROOT_LOG_DIRECTORY)
                .append("/").append(GeneratedValidator.ROOT_LOG_PREFIX)
                .append(hl7Templates2GOC.getGenerationDate());
        if(!validatorGenerated){
            throw new ValidatorGenerationException("Please refer to the logs at: "+ logPath.toString());
        }

        //Buil Generated Validator Object
        ValidatorConfiguration validatorConfiguration = hl7Templates2GOC.getValidatorConfiguration();
        ProjectNaming projectNaming = validatorConfiguration.getProjectNaming();
        String projectName = projectNaming.getProjectName();

        GeneratedValidator validator = new GeneratedValidator(outputFolder, hl7Templates2GOC.getGenerationDate(),projectName,bbr);

        // Setup Rights
        try {
            ValidatorGeneratorUtils.makeGeneratedValidatorExecutable(validator);
            System.out.println("Validator generated successfully");
        } catch (ProcessExecutionException | IOException e) {
            throw new ValidatorGenerationException("Failed to make generated validator executable",e);
        }

        return validator;

    }

    @Override
    public GeneratedValidator generateValidator(URL bbrURL) throws ValidatorGenerationException {

        // Get maven path
        String mvn = EnvironementConfiguration.getInstance().getProperty("maven.path");
        if(mvn == null){
            throw new ValidatorGenerationException("Maven path is not set in "+EnvironementConfiguration.APPLICATION_PROPERTIES_FILE);
        }

        // Check if GOC environment is setup
        Path path = EnvironementConfiguration.getInstance().getGocOutputFolder();
        if(path == null){
            throw new ValidatorGenerationException("GOC Output folder is not set");
        }

        // Get BBR String path from URL Resource
        String bbrPath;
        try {
            bbrPath = new File(bbrURL.toURI()).getAbsolutePath();
        } catch (URISyntaxException e) {
            throw new ValidatorGenerationException("Failed to load BBR from resources",e);
        }

        return generateValidator(bbrPath,path.toAbsolutePath().toString(),mvn);

    }
}
